local name, addon = ...
local parentName = addon['parentName'] or "BrokerPack"
local childName = "zz_GarrisonToggle"
local realm = GetRealmName()
local player = UnitName('player').."-"..realm
local temp = {}
local defaults = {
	['garrisonText'] = 'GS',
	['garrisonHideCombat'] = false,
	['hideStandard'] = true,
}

local options = {
}

local db

local function OnText(message)
	addon['OnText'](childName, message)
end
local function switchDisplay(value)
	if(value) then
		AlertFrame:UnregisterEvent("GARRISON_MISSION_FINISHED")
		AlertFrame:UnregisterEvent("GARRISON_BUILDING_ACTIVATABLE")
	else
		AlertFrame:RegisterEvent("GARRISON_MISSION_FINISHED")
		AlertFrame:RegisterEvent("GARRISON_BUILDING_ACTIVATABLE")	
	end
end
local function getPref(pref)
	return db[pref[#pref]]
end

local function setPref(pref,value)
	if(pref[#pref] == "garrisonText" and value=="") then
		value = nil
	elseif(pref[#pref] == "garrisonHideCombat") then
		switchDisplay(value)
	end
	db[pref[#pref]] = value
	OnText(db['garrisonText'] or GARRISON_LOCATION_TOOLTIP)
end
local function OnButton(self,button)
	if(button=="LeftButton") then
		GarrisonLandingPageMinimapButton:Click("LeftButton")
--		ToggleFrame(GarrisonLandingPage)
	elseif(button=="RightButton") then
		if(InterfaceOptionsFrame:IsVisible() and not InCombatLockdown()) then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(childName)
		end
	end
end

local function IsInProgress(mid)
	for k,v in pairs(C_Garrison.GetInProgressMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0)) do
		if(tonumber(v['missionID'])==tonumber(mid)) then
			return true
		end
	end
	return false
end

local function GetMission(missionID)
	wipe(temp)
--	C_Garrison.GetInProgressMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0)
	for k,v in pairs(C_Garrison.GetInProgressMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0)) do
		if(missionID==v['missionID']) then
			return v
		end
	end
end
local function AddDB(missionID)
	local missionInfo = GetMission(missionID)
	if(missionInfo) then
		local mt = {C_Garrison.GetMissionTimes(missionID)}
		db['missions'][player]['ml'][missionID] = time() + mt[5]
	end
end
local function CheckDB(missionID)
	local missionInfo = GetMission(missionID)
	if(missionInfo) then
		local old = db['missions'][player]['ml'][missionID]
		local mt = {C_Garrison.GetMissionTimes(missionID)}
		db['missions'][player]['ml'][missionID] = db['missions'][player]['ml'][missionID] or (time() + mt[5])
		if(db['missions'][player]['ml'][missionID] ~= old) then
			print("RESET",missionInfo['name'])
		end
	end
end
local function prune()
	if(#C_Garrison.GetInProgressMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0)<1) then return end
	local ti = time()
	for k,v in pairs(db['missions'][player]['ml']) do
		if(not IsInProgress(k) and db['missions'][player]['ml'][k] < ti) then
			db['missions'][player]['ml'][k] = nil
		end
	end
end

local function OnTooltipShow(GameTooltip)
	GameTooltip:AddLine(childName)
	local num = C_Garrison.GetNumFollowers(LE_FOLLOWER_TYPE_GARRISON_6_0)
	wipe(temp)
	C_Garrison.GetInProgressMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0)

	sort(temp,function(a,b)
		return (db['missions'][player]['ml'][a['missionID']] or 0) < (db['missions'][player]['ml'][b['missionID']] or 0)
	end)
	local ti = time()
	for k,v in pairs(temp) do
		CheckDB(tonumber(v['missionID']))
		GameTooltip:AddDoubleLine(v['name'],db['missions'][player]['ml'][tonumber(v['missionID'])]>ti and v['timeLeft'] or addon['colorize']("Done","00ff00"))
		num = num - v['numFollowers']
	end
	for k, v in pairs(db['missions']) do
		if(k ~= player) then
			for a,b in pairs(v['ml']) do
				if(b<ti) then
					GameTooltip:AddDoubleLine(k, addon['colorize']("Mission done","00ff00"))
					break
				end
			end
		end
	end
	if(IsShiftKeyDown()) then
		for k, v in pairs(db['missions']) do
			local found = 0
			for a,b in pairs(v['ml']) do
				if(b>=ti) then
					local r = b - ti
					if(r<found or found==0) then
						found = r
					end
				end
			end
			if(found>0) then
				GameTooltip:AddDoubleLine(k, addon['colorize'](format("Next %.1f Min",found / 60 ),"ffff00"))
			end
		end
	end
	GameTooltip:AddDoubleLine("Follower:",num.."/"..C_Garrison.GetNumFollowers(LE_FOLLOWER_TYPE_GARRISON_6_0))
	GameTooltip:AddDoubleLine("Missions:",#C_Garrison.GetInProgressMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0).."/"..#C_Garrison.GetAvailableMissions(temp, LE_FOLLOWER_TYPE_GARRISON_6_0))
	GameTooltip:AddLine(addon['colorize']("Press SHIFT for more info","00ff00"))
end
local function event(self,event,missionID,...)
	if(event == "ADDON_LOADED" and missionID~="Blizzard_GarrisonUI") then
		self:UnregisterEvent(event)
		if(db['hideStandard']) then
			GarrisonLandingPageMinimapButton:UnregisterAllEvents()
--			GarrisonLandingPageMinimapButton:SetScript("OnClick",nil)
--			GarrisonLandingPageMinimapButton:SetScript("OnEvent",nil)
--			GarrisonLandingPageMinimapButton:SetScript("OnShow",nil)
			GarrisonLandingPageMinimapButton:Hide()
		end
		return
	end
	if(event == "GARRISON_MISSION_COMPLETE_RESPONSE") then
		local canComplete, succeeded = ...
		if(canComplete) then
--			db['missions'][player]['ml'][missionID] = nil
		end
	elseif(event == "GARRISON_MISSION_STARTED") then
		AddDB(missionID)
	end
	prune()
end
local function init()
	db = addon['db']['profile'][childName]
	db['missions'] = db['missions'] or {}
	db['missions'][player] = db['missions'][player] or {
		['ml'] = {}
	}
	options = addon['InitConfig'](addon, childName, true, {
		['type'] = "launcher",
		['OnClick'] = OnButton,
		['text'] = db['garrisonText'] or GARRISON_LOCATION_TOOLTIP,
		['OnTooltipShow'] = OnTooltipShow,
		['icon'] = "Interface\\Icons\\Achievement_Arena_2v2_6",
	}, getPref, setPref)
	addon['AddConfigEntry'](addon,childName,'toggle','garrisonHideCombat',"Hide Alert","Hide mission complete alert",1,nil,nil,nil,nil)
	addon['AddConfigEntry'](addon,childName,'toggle','hideStandard',"Hide Blizzard","Hide Blizzards default garrison button",1,nil,nil,nil,nil)
	addon['AddConfigEntry'](addon,childName,'input','garrisonText',"Display Text","",1,nil,nil,nil,nil)
	addon['RegisterFunc']({"GARRISON_MISSION_STARTED","GARRISON_MISSION_FINISHED","GARRISON_MISSION_COMPLETE_RESPONSE","ADDON_LOADED","PLAYER_LOGOUT"},"OnEvent", event)
	if(not IsAddOnLoaded("Blizzard_GarrisonUI")) then
		UIParentLoadAddOn("Blizzard_GarrisonUI")
	end
	switchDisplay(db['garrisonHideCombat'])
end
addon['startup'](addon, name, childName, init, true, defaults)